﻿using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Media.Imaging;

using DevExpress.Data;
using DevExpress.Mvvm;
using DevExpress.Mvvm.UI;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 알림 서비스
        /// </summary>
        private NotificationService service;

        /// <summary>
        /// 비트맵 이미지
        /// </summary>
        private BitmapImage bitmapImage;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            this.service = new NotificationService();

            this.service.ApplicationId                   = "TestProject";
            this.service.ApplicationName                 = "TestProject";
            this.service.UseWin8NotificationsIfAvailable = true;
            this.service.CreateApplicationShortcut       = true;
            this.service.PredefinedNotificationDuration  = PredefinedNotificationDuration.Default;
            this.service.PredefinedNotificationTemplate  = NotificationTemplate.ShortHeaderAndTwoTextFields;

            ShellHelper.TryCreateShortcut(this.service.ApplicationId, this.service.ApplicationName);

            Uri uri = GetResourceURI(null, "IMAGE/toast.png");

            this.bitmapImage = new BitmapImage(uri);

            Closing                                += Window_Closing;
            this.showToastNotificationButton.Click += showToastNotificationButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 윈도우 닫을 경우 처리하기 - Window_Closing(sender, e)

        /// <summary>
        /// 윈도우 닫을 경우 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void Window_Closing(object sender, CancelEventArgs e)
        {
            ShellHelper.TryRemoveShortcut(this.service.ApplicationName);
        }

        #endregion
        #region 토스트 알림 표시하기 버튼 클릭시 처리하기 - showToastNotificationButton_Click(sender, e)

        /// <summary>
        /// 토스트 알림 표시하기 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void showToastNotificationButton_Click(object sender, RoutedEventArgs e)
        {
            INotification notification = this.service.CreatePredefinedNotification
            (
                "알림",
                "등록된 사용자가 아닙니다.",
                DateTime.Now.ToString(),
                this.bitmapImage
            );

            notification.ShowAsync().ContinueWith(resultTask => ProcessToastNotificationClicked(resultTask, null));
        }

        #endregion
        #region 토스트 알림 클릭시 처리하기 - ProcessToastNotificationClicked(resultTask, parameter)

        /// <summary>
        /// 토스트 알림 클릭시 처리하기
        /// </summary>
        /// <param name="resultTask">결과 태스크</param>
        /// <param name="parameter">매개 변수</param>
        private void ProcessToastNotificationClicked(Task<NotificationResult> resultTask, object parameter)
        {
            NotificationResult result = resultTask.Result;

            switch(result)
            {
                case NotificationResult.Activated :

                    Process.Start("https://icodebroker.tistory.com");

                    break;
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 리소스 URI 구하기 - GetResourceURI(assemblyName, resourcePath)

        /// <summary>
        /// 리소스 URI 구하기
        /// </summary>
        /// <param name="assemblyName">어셈블리명</param>
        /// <param name="resourcePath">리소스 경로</param>
        /// <returns>리소스 URI</returns>
        private Uri GetResourceURI(string assemblyName, string resourcePath)
        {
            if(string.IsNullOrEmpty(assemblyName))
            {
                return new Uri($"pack://application:,,,/{resourcePath}");
            }
            else
            {
                return new Uri($"pack://application:,,,/{assemblyName};component/{resourcePath}");
            }
        }

        #endregion
    }
}